;
#include "p16F690.inc"
;
; CONFIG
; __config 0x30C4
 __CONFIG _FOSC_INTRCIO & _WDTE_OFF & _PWRTE_ON & _MCLRE_OFF & _CP_OFF & _CPD_OFF & _BOREN_OFF & _IESO_OFF & _FCMEN_OFF
;
;
;<editor-fold defaultstate="collapsed" desc="vars">
	cblock	0x20	;	ram 0x20 - 0x7f
	temp
	temp1
	temp2
	temp_hi
	temp_lo
	comm_state	; commutation state 0-5
	ct_hi		; commutation time
	ct_lo
	zc_hi		; captured zero-cross time
	zc_lo
	mode		; current mode:
				; 0 -
				; 1 -
				; 2 -
				; 3 -
				; 4 -
				; 5	-
				; 6 -
				; 7 -
	lost_count
	endc
;</editor-fold>
;
;----------------------------------------------------------
;
	org	0
	nop
	goto	init
;
	org	4h
;
;<editor-fold defaultstate="collapsed" desc="int_handler">
;
; TMR1 and CM2 int
;
	bcf	status,rp0
	bcf	status,rp1
;
	btfsc	pir1,t1if
	goto	t1_int
	btfsc	pir2,c2if
	goto	c2_int
	retfie
;
;<editor-fold defaultstate="collapsed" desc="timer1_int">
;
t1_int
	bcf	pir2,c2if
	bcf	t1con,tmr1on
	movf	ct_hi,w		; tmr1=ct
	movwf	tmr1h
	movf	ct_lo,w
	movwf	tmr1l
	bcf	pir1,t1if
	bsf	t1con,tmr1on
	call	adc
	call	commutate
;
	bcf	portb,7
	btfss	mode,0				; if BEMF flag
	goto	t1_math				; skip math
;
	call	bemf_blanking		; zc event expected
	bcf	mode,0
;
	retfie
;
t1_math
	incf	lost_count,f
	movlw	0x40
	btfsc	lost_count,1		; reset	ct if zc not detected
	movwf	ct_hi
	btfss	mode,1				; skip math if zc not detected
	retfie
	bcf	mode,1
	clrf	lost_count
;
	comf	ct_hi,w				; temp = ct_time/2
	movwf	temp_hi
	comf	ct_lo,w
	movwf	temp_lo
	bcf	status,c
	rrf	temp_hi,f
	rrf	temp_lo,f
	comf	temp_hi,f
	comf	temp_lo,f
;
	movf	temp_lo,w		; temp = zc - ct_time/2 = zc_error
	subwf	zc_lo,w
	movwf	temp_lo
	btfss	status,c
	incf	temp_hi,f
	movf	temp_hi,w
	subwf	zc_hi,w
	movwf	temp_hi
;
	btfsc	status,c		; check for negative
	goto	t1_math_1
;
	bsf	portb,5
	comf	temp_hi,f		; ct = ct + error
	comf	temp_lo,f		; acceleration
	bcf	status,c			; scaling factor
	rrf	temp_hi,f
	rrf	temp_lo,f
	bcf	status,c
	rrf	temp_hi,f
	rrf	temp_lo,f
	bcf	status,c
	rrf	temp_hi,f
	rrf	temp_lo,f
	bsf	temp_lo,0			; minimum error
	movf	temp_lo,w
	addwf	ct_lo,f
	btfsc	status,c
	incf	ct_hi,f
	movf	temp_hi,w
	addwf	ct_hi,f
	retfie
;
t1_math_1
;
	bcf	portb,5
	bcf	status,c		; ct = ct - error
	rrf	temp_hi,f		; decceleration
	rrf	temp_lo,f
	bcf	status,c
	rrf	temp_hi,f
	rrf	temp_lo,f
	bcf	status,c
	rrf	temp_hi,f
	rrf	temp_lo,f
	bsf	temp_lo,0
	movf	temp_lo,w
	subwf	ct_lo,f
	btfss	status,c
	incf	temp_hi,f
	movf	temp_hi,w
	subwf	ct_hi,f
	retfie
;
;</editor-fold>
;
;<editor-fold defaultstate="collapsed" desc="comparator_int">
c2_int
;
	bcf	pir2,c2if
	goto	c2_end
;
	clrf	temp			; check if not single pike
	clrf	temp1
	bsf	status,irp
	movlw	low cm2con0
	movwf	fsr
c2_check					; 8cmd one run
	btfsc	indf,c2out		; or 4*count us
	incf	temp,f
	incf	temp1,f
	movlw	.50				; number of samples
	subwf	temp1,w
	btfss	status,c
	goto	c2_check
	bcf	status,irp
;
	bcf	pir2,c2if			; should be present in at least of
	bcf	status,c			; half samples
	rrf	temp1,w
	subwf	temp,w
	btfss	status,c
	retfie
;
c2_end
	bsf	portb,7				; zc event detected
	bsf	mode,1
;
	bcf	t1con,tmr1on
	movf	tmr1h,w			; zc=tmr1
	movwf	zc_hi
	movf	tmr1l,w
	movwf	zc_lo
	bsf	t1con,tmr1on
;
	bsf	status,rp1			; comp off
	movlw	0x00
	movwf	cm2con0
	bcf	status,rp1
;
	bcf	pir2,c2if
;
	retfie
;
;</editor-fold>
;
;</editor-fold>
;
;==========================================================
;
;<editor-fold defaultstate="collapsed" desc="adc">
adc
	btfsc	adcon0,1
	return
	movf	adresh,w
	movwf	temp
	rrf	temp,f
	rrf	temp,w
	andlw	b'00111111'
	addlw	.30
	movwf	ccpr1l
	bsf	adcon0,1
	return
;
;</editor-fold>
;
;----------------------------------------------------------
;
;<editor-fold defaultstate="collapsed" desc="commutate">
;
commutate
;
	movf	comm_state,w
	andlw	0x07
	addwf	pcl,f
	goto	comm_0
	goto	comm_1
	goto	comm_2
	goto	comm_3
	goto	comm_4
	goto	comm_5
	nop
	nop
	return
;
;	motor port table
;	porta		{0x04,0x04,0x10,0x10,0x20,0x20}
;	ccp1con		{0xcc,0x4e,0x4e,0x0c,0x0c,0xcc}
;	cm2con0		{0x83,0x00,0x80,0x00,0x81,0x00}
;
comm_0
;
	clrf	porta
	nop
	movlw	0x04
	movwf	porta
;
	bsf	status,rp1
	movlw	0x83
	movwf	cm2con0
	bcf	status,rp1
;
	movlw	.1
	movwf	comm_state
	bsf	mode,0			; BEMF sensing flag
	return
;
comm_1
;
	movlw	0x4e
	movwf	ccp1con
	movlw	.59
	movwf	tmr2
;
	bsf	status,rp1
	movlw	0x00
	movwf	cm2con0
	bcf	status,rp1
;
	movlw	.2
	movwf	comm_state
	return
;
comm_2
;
	clrf	porta
	nop
	movlw	0x10
	movwf	porta
;
	bsf	status,rp1
	movlw	0x80
	movwf	cm2con0
	bcf	status,rp1
;
	movlw	.3
	movwf	comm_state
	bsf	mode,0
	return
;
comm_3
;
	movlw	0x0c
	movwf	ccp1con
	movlw	.59
	movwf	tmr2
;
	bsf	status,rp1
	movlw	0x00
	movwf	cm2con0
	bcf	status,rp1
;
	movlw	.4
	movwf	comm_state
	return
;
comm_4
;
	clrf	porta
	nop
	movlw	0x20
	movwf	porta
;
	bsf	status,rp1
	movlw	0x81
	movwf	cm2con0
	bcf	status,rp1
;
	movlw	.5
	movwf	comm_state
	bsf	mode,0
	return
;
comm_5
;
	movlw	0xcc
	movwf	ccp1con
	movlw	.59
	movwf	tmr2
;
	bsf	status,rp1
	movlw	0x00
	movwf	cm2con0
	bcf	status,rp1
;
	movlw	.0
	movwf	comm_state
	return
;
;</editor-fold>
;
;----------------------------------------------------------
;
;<editor-fold defaultstate="collapsed" desc="bemf_blanking">
bemf_blanking			; wait until transient on free motor terminal
	clrf	temp		; or ~500us max at 8MHz
	bsf	status,irp
	movlw	low cm2con0
	movwf	fsr
bemf_wait
	incf	temp,f
	btfsc	status,z
	goto	bemf_end
	btfsc	indf,c2out
	goto	bemf_wait
bemf_end
	bcf	status,irp
	bcf	pir2,c2if
	return
;
;</editor-fold>
;
;==========================================================
;
;<editor-fold defaultstate="collapsed" desc="init">
init
;
	clrf	porta
	clrf	portb
	clrf	portc
;
	bsf	status,rp0
	movlw	b'00000011'
	movwf	trisa
	movlw	b'00000000'
	movwf	trisb
	movlw	b'00001011'
	movwf	trisc
	movlw	b'01110001'
	movwf	osccon
	movlw	b'10001000'
	movwf	option_reg
;
	movlw	b'01110000'
	movwf	adcon1
	movlw	.59
	movwf	pr2
	bsf	pie1,0
	bsf	pie2,6
;
	bsf	status,rp1
	movlw	0x80
	movwf	cm2con0
	movlw	b'10110011'
	movwf	ansel
	movlw	b'00000010'
	movwf	anselh
	bcf	status,rp1
;
	bcf	status,rp0
	movlw	b'00000001'
	movwf	adcon0
	movlw	b'00000001'
	movwf	t1con
	movlw	b'00000100'
	movwf	t2con
	movlw	b'00001100'
	movwf	ccp1con
	clrf	tmr1h
	clrf	tmr1l
	movlw	.90
	movwf	ccpr1l
;
;	bcf	pir1,0				; sets rotor to determined position
;	call	comm_5
;	btfss	pir1,0
;	goto	$-1
;	bcf	pir1,0
;	btfss	pir1,0
;	goto	$-1
;	bcf	pir1,0
;	movlw	b'00000001'
;	movwf	t1con
;
	clrf	mode
	clrf	comm_state
	clrf	temp1
	movlw	0x40
	movwf	ct_hi
;	movwf	ct_lo
	clrf	ct_lo
;
	movlw	b'11000000'
	movwf	intcon
;
	goto	$
;
;</editor-fold>
;
	end
;